<?php
namespace AustraliaPost\Extensions\Business\API\Wrapper;

use WP_Error;

class Connector {

	const API_URL_TEST_MODE = 'https://digitalapi.auspost.com.au/test/shipping/v1';
	const API_URL_LIVE_MODE = 'https://digitalapi.auspost.com.au/shipping/v1';
	const PARTNER_ID = 'WPRUBY-8882';

	private $username;
	private $password;
	private $account_number;
	/**
	 * @var bool
	 */
	private $test_mode;


	/**
	 * Connector constructor.
	 *
	 * @param string $username
	 * @param string $password
	 * @param string $account_number
	 * @param boolean $test_mode
	 */
	public function __construct( $username, $password, $account_number, $test_mode )
	{
		$this->username       = $username;
		$this->password       = $password;
		$this->account_number = $account_number;
		$this->test_mode = false;
	}


	/**
	 * @param $endpoint
	 * @param $method
	 * @param null $payload
	 *
	 * @return array
	 */
	public function connect($endpoint, $method, $payload = null){
		$headers = $this->getHeaders();
		$arguments = [
			'method' => strtoupper($method),
			'headers' => $headers,
		];

		if ($payload !== null ){
			$arguments['body'] = $payload;
		}

		$api_url = (true === $this->test_mode)? self::API_URL_TEST_MODE: self::API_URL_LIVE_MODE;

		$response = wp_remote_request( $api_url . $endpoint, $arguments );
		return $this->handleResponse($response);
}

	/**
	 * @param $response
	 *
	 * @return array|string
	 */
	private function handleResponse( $response ) {

		//error
		if ($response instanceof WP_Error){
			return array($response->get_error_code() => $response->get_error_message());
		}
		$header = $response['headers'];
		if ($header['content-type'] === 'application/pdf') {
			return wp_remote_retrieve_body($response);
		}

		$responseBody = json_decode(wp_remote_retrieve_body($response), true);

		if (isset($responseBody['errors'])){
			return ['errors' => $responseBody['errors']];
		}

		//success
		return $responseBody;

	}

	private function getHeaders() {
		return [
			'Accept'         => 'application/json',
			'Content-Type'   => 'application/json',
			'Account-Number' => $this->account_number,
			'Authorization'  => sprintf('Basic %s', $this->getAuthString()),
			'Partner-ID'     => self::PARTNER_ID
		];
	}

	private function getAuthString() {
		return base64_encode(sprintf('%s:%s', $this->username, $this->password));
	}

	/**
	 * @return mixed
	 */
	public function getAccountNumber() {
		return $this->account_number;
	}


}
